import React from 'react';
import Taro from '@tarojs/taro';
import classNames from 'classnames';
import { View } from '@tarojs/components';
import cls from './style.module.less';

export default function TreeItem(props) {
  const { data, expanded, onClick, onToggle, renderTitle, onHeight } = props;
  const ctt = React.useMemo(() => `ctt-${Math.random().toString(36).substring(2, 7)}`, []);
  const [current, setCurrent] = React.useState(-1);
  const [height, setHeight] = React.useState(0);
  const expandedRef = React.useRef(false);
  expandedRef.current = expanded;

  const style = {
    maxHeight: expanded ? `${height + 10}px` : '0px',
  }
  
  const handleToggle = (e) => {
    e.stopPropagation();
    e.preventDefault();
    const checked = !expandedRef.current;
    onToggle && onToggle(checked);
    onClick && onClick(data);
    onHeight && onHeight();
  }

  const handleChildToggle = (e, inx) => {
    setCurrent(e ? inx : -1);
  }

  const calcHeight = React.useCallback(() => {
    Taro.nextTick(() => {
      Taro.createSelectorQuery()
      .select(`.${ctt}`)
      .boundingClientRect(res => {
        // console.log('-----rect------', ctt, res?.height)
        setHeight(res?.height || 0);
      })
      .exec();
    })
  }, []);

  React.useEffect(() => {
    calcHeight();
  }, []);

  return (
    <View className={classNames(cls.item, {[cls.expanded]: expanded})} onClick={handleToggle}>
      {/* <View className={cls.toggle} onClick={handleToggle}></View> */}
      {renderTitle ? renderTitle(data) : null}
      {
        data?.children?.length > 0 && (
          <View class={classNames(cls.children)} style={style}>
            <View className={ctt}>
            {
              data.children.map((it, inx) => (
                <TreeItem
                  key={inx}
                  data={it}
                  expanded={current == inx}
                  onClick={onClick}
                  onToggle={(e) => handleChildToggle(e, inx)}
                  onHeight={onHeight || calcHeight}
                  renderTitle={renderTitle}
                />
              ))
            }
            </View>
          </View>
        )
      }
    </View>
  );
}
